package cn.gygxzc.envir.sys.service.impl

import cn.gygxzc.envir.sys.dao.IUserDao
import cn.gygxzc.envir.sys.model.SysUser
import cn.gygxzc.envir.sys.service.ILoginService
import cn.gygxzc.tina.config.jwt.JwtConfig
import cn.gygxzc.tina.config.jwt.JwtPayload
import cn.gygxzc.tina.config.jwt.JwtUtils
import cn.gygxzc.tina.config.shiro.cache.JwtRedisCache
import cn.gygxzc.tina.config.shiro.common.ZenAuthToken
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper
import org.apache.shiro.SecurityUtils
import org.apache.shiro.authc.AuthenticationException
import org.apache.shiro.session.mgt.SimpleSession
import org.apache.shiro.subject.support.DelegatingSubject
import org.apache.shiro.web.subject.support.WebDelegatingSubject
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * Created by niantuo on 2018/9/21.
 * 实际上客户端只能从redis缓存中去取数据，也就是说
 * 登陆需要在授权服务器，授权服务器会将用户数据存入redis缓存
 * 客户端可以获取用户数据，如果redis没有改数据，那么表示该用户没有登录
 */

@Service
class LoginService : ILoginService {

    private val logger = LoggerFactory.getLogger(LoginService::class.java)

    @Autowired
    private lateinit var userDao: IUserDao

    @Autowired
    private lateinit var jwtConfig: JwtConfig

    @Autowired
    private lateinit var redisCache: JwtRedisCache


    /**
     * 登陆之后将用户信息保存到redis缓存
     */
    override fun login(username: String, password: String): String {
        val wrapper = QueryWrapper<SysUser>()
        wrapper.eq("account", username)

        logger.info("login select user : ${wrapper.expression}")

        val user = userDao.selectOne(wrapper)
        val authToken = ZenAuthToken(username, password, user)
        val subject = SecurityUtils.getSubject() as WebDelegatingSubject
        try {
            subject.login(authToken)
            redisCache.put(username, authToken)
            return JwtUtils.sign(JwtPayload(user.id.toString(), user.account!!), jwtConfig)
        } catch (e: AuthenticationException) {
            throw e
        }

    }
}