package cn.gygxzc.tina.config.shiro.utils

import com.gxzc.zen.api.sys.common.MenuTree
import com.gxzc.zen.api.sys.model.SysRole
import cn.gygxzc.envir.sys.model.SysUser
import com.gxzc.zen.api.sys.service.IPermissionService
import com.gxzc.zen.common.util.SpringContextHolder
import cn.gygxzc.tina.config.shiro.common.ZenPermission
import cn.gygxzc.tina.config.shiro.ZenHttpSession
import org.apache.shiro.SecurityUtils
import org.apache.shiro.session.Session
import org.slf4j.LoggerFactory

/**
 * SSO 工具类
 * @author NorthLan
 * @date 2018/3/15
 * @url https://noahlan.com
 */
@Suppress("UNCHECKED_CAST")
object SSOUtil {
    private var permissionService: IPermissionService? = SpringContextHolder.getBean(IPermissionService::class.java)
    private val logger = LoggerFactory.getLogger(SSOUtil::class.java)
    private const val NA = "NA"

    fun getSession(): Session? {
        return SecurityUtils.getSubject().getSession(false)
    }

    fun setAttribute(key: Any, value: Any?) {
        getSession()?.setAttribute(key, value)
    }

    fun isLogin(): Boolean {
        return SecurityUtils.getSubject()?.isAuthenticated ?: false
    }

    /**
     * 获取当前登陆 账号
     */
    fun getCurAccount(): String {
        return try {
            SecurityUtils.getSubject().principal.toString()
        } catch (e: Throwable) {
            logger.warn("get curAccount error, return empty string:\"\"")
            ""
        }
    }

    /**
     * 获取当前登陆 userId
     */
    fun getCurUserId(): Long {
        return try {
            getSession()?.getAttribute(ZenHttpSession.SESSION_KEY_USER_ID) as Long
        } catch (e: Throwable) {
            logger.warn("getCurUserId error, return -1")
            -1
        }
    }

    /**
     * 获取当前登陆 基本信息
     */
    fun getCurUserInfo(): SysUser? {
        return try {
            getSession()?.getAttribute(ZenHttpSession.SESSION_KEY_USER_INFO) as? SysUser
        } catch (e: Throwable) {
            logger.warn("get curUserInfo error, return null")
            null
        }
    }

    /**
     * 获取当前登陆用户名
     */
    fun getCurUserName(): String {
        return try {
            getCurUserInfo()?.username!!
        } catch (e: Throwable) {
            logger.warn("getCurUserName error, return given NA: \"NA\" ")
            ""
        }
    }

    /**
     * 获取当前登陆 角色列表
     */
    fun getCurUserRoles(): MutableList<SysRole>? {
        return getSession()?.getAttribute(ZenHttpSession.SESSION_KEY_USER_ROLES) as? MutableList<SysRole>
    }

    /**
     * 获取当前用户 权限列表
     */
    fun getCurUserPerms(): MutableList<ZenPermission>? {
        if (permissionService == null) {
            permissionService = SpringContextHolder.getBean(IPermissionService::class.java)
        }
        return permissionService!!.getSubjectPermission()
    }

    /**
     * 获取当前用户 目录树
     */
    fun getCurUserMenuTree(): MutableList<MenuTree>? {
        return getSession()?.getAttribute(ZenHttpSession.SESSION_KEY_USER_MENU) as? MutableList<MenuTree>
    }
}